﻿#!/usr/local/bin/python
# -*- coding: utf-8 -*-

import ssl
import sys
import urllib2
from urlparse import parse_qsl

from resources import static, libfilmux as filmux
import xbmc
import xbmcaddon
import xbmcgui
import xbmcplugin

_url = sys.argv[0]
_handle = int(sys.argv[1])
_addon = xbmcaddon.Addon()
_name = _addon.getAddonInfo('name')
_icon = _addon.getAddonInfo('icon')


def play_video(link):
    """
    Play a video.
    """
    play_item = xbmcgui.ListItem(path=link)
    play_item.setInfo(type="video", infoLabels=static.VideoInfo.load_current().compose())
    xbmcplugin.setResolvedUrl(_handle, True, listitem=play_item)


def list_link(link=None, add=None, menus=static.main_menu):
    """
    Opens a regular menu.
    """
    if link is not None:
        for menu_no in link.split('.'):
            menus = menus[int(menu_no)].option
    listing = []
    for entry, menu in enumerate(menus):
        list_item = xbmcgui.ListItem(label=menu.label, thumbnailImage=menu.image)
        list_item.setProperty('fanart_image', menu.image)
        additional = add or {}
        if menu.additional:
            additional.update(menu.additional)

        if menu.type is static.Menu.Menu:
            link0 = entry if link is None else "{}.{}".format(link, entry)
            url = '{0}?action=list&link={1}'.format(_url, link0)
        elif menu.type is static.Menu.Link:
            url = '{0}?action=open&link={1}'.format(_url, menu.option)
        elif menu.type is static.Menu.Search:
            url = '{0}?action=find'.format(_url)
        elif menu.type is static.Menu.Top:
            url = '{0}?action=top100'.format(_url)
        elif menu.type is static.Menu.Categories:
            url = '{0}?action=categories'.format(_url)
        else:
            continue

        for k, v in additional.items():
            url += '&{}={}'.format(k, v)
        listing.append((url, list_item, True))
    xbmcplugin.addDirectoryItems(_handle, listing, len(listing))
    xbmcplugin.endOfDirectory(_handle)


def open_link(data, paging=None, add=None):
    """
    Opens videos menu & renders list of available videos.
    :type data: videos
    :type paging: paging
    """
    if not data:
        xbmcgui.Dialog().notification(_name, "Nieko nerado", icon=xbmcgui.NOTIFICATION_INFO)
        return

    listing = []
    for video in data:
        title = video.get('title')
        if video.get('added'):
            title = u"{} [COLOR red]{}[/COLOR]".format(title, video.get('added'))
        list_item = xbmcgui.ListItem(title, thumbnailImage=video.get('img'))
        list_item.setInfo(type="video", infoLabels={
            'title': video.get('title'),
            'year': video.get('year'),
            'genre': video.get('gendre'),
            'plot': video.get('description'),
        })
        if video.get('language'):
            list_item.addStreamInfo("audio", {'language': video.get('language')})
        # if video.codec:
        #     list_item.addStreamInfo('video', {'codec': video.codec})
        url = '{0}?action=view&link={1}'.format(_url, video.get('url'))
        listing.append((url, list_item, True))
    if paging and paging.get('next_url'):
        next_url = '{0}?action=open&link={1}'.format(_url, paging.get('next_url'))
        if add:
            for k, v in add.items():
                next_url += '&{}={}'.format(k, v)
        list_item = xbmcgui.ListItem(label="[COLOR blue]Sekantis (Puslapis {} iš {})[/COLOR]"
                                     .format(paging.get('current', 1)+1, paging.get('total', 2)))
        listing.append((next_url, list_item, True))
    xbmcplugin.addDirectoryItems(_handle, listing, len(listing))
    xbmcplugin.endOfDirectory(_handle)


def open_video(link):
    """
    Open video page, grabs available video info and streams.
    :param link: Video page link
    """
    data = filmux.process_video(link)
    if data.get('trailer'):
        try:
            s = data.get('trailer')
            video_id = s[s.index('/embed/') + 7:s.index('?')]
            art = "https://img.youtube.com/vi/{}/0.jpg".format(video_id)
            list_item = xbmcgui.ListItem(label="Pristatymas", thumbnailImage=data.get('img'))
            list_item.setProperty('fanart_image', data.get('img').replace('/thumb/', ''))
            if xbmc.getCondVisibility('System.HasAddon(plugin.video.youtube)') == 1:
                url = 'plugin://plugin.video.youtube/play/?video_id={}'.format(video_id)
                list_item.setProperty('IsPlayable', 'true')
                list_item.setInfo(type="video", infoLabels={})
            else:
                url = '{0}?action=dialog&text={1}'.format(_url, "Nerasta įrengto youtube!")
            xbmcplugin.addDirectoryItem(_handle, url, listitem=list_item)
        except ValueError:
            pass
    for i, src in enumerate(data.get('sources', [])):
        # url = '{0}?action=play&video={1}'.format(_url, src.url)
        name = src.get('name')
        if not name:
            name = u'Šaltinis #%d' % i
        # res = None
        # if '360' in src.get('url'):
        #     res = '360p'
        # elif '480' in src.get('url'):
        #     res = '480p'
        # elif '720' in src.get('url'):
        #     res = '720p'
        # elif '1080' in src.get('url'):
        #     res = '1080p'
        # if res:
        #     name += ' [COLOR red]1080p[/COLOR]'
        list_item = xbmcgui.ListItem(name, thumbnailImage=src.get('img'))
        list_item.setProperty('fanart_image', data.get('img'))
        # list_item.setInfo(type="video", infoLabels=src.info.compose())
        #if data.lang:
        #     list_item.addStreamInfo("audio", {'language': data.lang})
        # if data.subtitle:
        #     list_item.addStreamInfo("subtitle", {'language': data.subtitle})
        list_item.setProperty('IsPlayable', 'true')
        # listing.append((src.url, list_item, False))
        xbmcplugin.addDirectoryItem(_handle, src.get('url'), listitem=list_item)
    xbmcplugin.endOfDirectory(_handle)


def router(paramstring):
    """
    Router function that calls other functions
    depending on the provided paramstring
    """
    # Parse a URL-encoded paramstring to the dictionary of
    # {<parameter>: <value>} elements
    params = dict(parse_qsl(paramstring))
    if params:
        additional = params.copy()
        for opt in ['action', 'link', 'video', 'page', 'name', 'text']:
            if opt in additional:
                del additional[opt]

        if params['action'] == 'list':
            # Display the list of menus in a provided in static.py.
            list_link(params['link'], additional)

        elif params['action'] == 'open':
            # Generate list from website.
            videos, paging = filmux.process_page(params['link'])  # , params['sort'] if 'sort' in params else None
            open_link(videos, paging=paging, add=additional)

        elif params['action'] == 'view':
            # Open video menu (form source).
            open_video(params['link'])

        elif params['action'] == 'play':
            # Play a video from a provided URL.
            play_video(params['video'])

        elif params['action'] == 'find':
            # Do a search command
            if 'page' in params:
                page = int(params['page'])
                text = params['name']
            else:
                page = 0
                text = urllib2.quote(xbmcgui.Dialog().input('Įveskite paieškos frazę'))
            data, paging = filmux.search(text)
            url = '{0}?action=find&name={1}&page={2}'.format(_url, text, page + 1)
            # open_link(data, paging, url)
            open_link(data)

        # elif params['action'] == 'top100':
        #      open_link(static.process_top100())
        #
        # elif params['action'] == 'categories':
        #      list_link(menus=static.process_categories())

        elif params['action'] == 'dialog':
            xbmcgui.Dialog().notification(_name, params['text'], icon=xbmcgui.NOTIFICATION_WARNING)

    else:
        # If the plugin is called from Kodi UI without any parameters,
        # display the list of video categories
        list_link()


if __name__ == '__main__':
    try:
        router(sys.argv[2][1:])
    except urllib2.HTTPError as e:
        xbmcgui.Dialog().notification(_name, "Jungimosi klaida {}".format(e.code))

    except urllib2.URLError as e:
        xbmcgui.Dialog().notification(_name, "Nepavyko prisijungti.", str(e.reason))

    except ssl.SSLError as e:
        if e.message == 'The read operation timed out':
            xbmcgui.Dialog().notification(_name, "Filmux.org puslapis negražino atsakymo", "Bandykite vėliau",
                                          icon=xbmcgui.NOTIFICATION_ERROR)
        else:
            xbmcgui.Dialog().notification(_name, "Jungimosi klaida!", e.message)

    except static.UpdateException as e:
        xbmcgui.Dialog().notification(_name, "Puslapio klaida", e.message)
